from pathlib import Path
from typing import Optional

from flake8.formatting.base import BaseFormatter
from flake8.options.manager import OptionManager
from flake8.violation import Violation


class GithubAnnotationsFormatter(BaseFormatter):
    def format(self, error: Violation) -> Optional[str]:
        return (
            f"::error file={self.path_prefix / error.filename},line={error.line_number}"
            f",col={error.column_number},title={error.code}::{error.text}"
        )

    @classmethod
    def add_options(cls, option_manager: OptionManager) -> None:
        option_manager.add_option(
            "--github-annotation-path-prefix",
            default="",
            parse_from_config=True,
            help=(
                "Path prefix to add to prepend to filenames"
                "when using --format github"
            ),
        )

    @classmethod
    def parse_options(cls, option_manager: OptionManager) -> None:
        cls.path_prefix = Path(option_manager.github_annotation_path_prefix)



import subprocess
import sys
import tempfile
from pathlib import Path
from typing import Generator

import pytest


@pytest.fixture
def file_with_error() -> Generator[str, None, None]:
    """Yields a path to a temprary file that contains a F401 error"""
    with tempfile.NamedTemporaryFile("w+") as f:
        f.write("import foo\n")
        f.flush()
        yield f.name


def test_output_is_formated_as_github_error_annotation(file_with_error: str):
    output = subprocess.run(
        [
            sys.executable,  # This makes sure flake8 is ran from the correct venv
            "-m",
            "flake8",
            file_with_error,
            "--format",
            "github",
        ],
        capture_output=True,
    )
    expected_output = f"::error file={file_with_error},line=1,col=1,title=F401::'foo' imported but unused\n"  # noqa:E501
    assert output.stdout.decode() == expected_output


def test_path_prefix_argument(file_with_error: str):
    path_prefix = Path("foo")
    output = subprocess.run(
        [
            sys.executable,  # This makes sure flake8 is ran from the correct venv
            "-m",
            "flake8",
            file_with_error,
            "--format",
            "github",
            "--github-annotation-path-prefix",
            path_prefix,
        ],
        capture_output=True,
    )

    expected_output = f"::error file={path_prefix / file_with_error},line=1,col=1,title=F401::'foo' imported but unused\n"  # noqa:E501
    assert output.stdout.decode() == expected_output



import sys

if sys.version_info < (3, 10):
    from importlib_metadata import entry_points
else:
    from importlib.metadata import entry_points


def test_entrypoint_registered():
    discovered_plugins = entry_points(group="flake8.report")
    matching_plugins = [
        plugin for plugin in discovered_plugins if plugin.name == "github"
    ]
    assert len(matching_plugins) == 1
